/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").  
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at sunpinyin/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at sunpinyin/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "ic_history.h"

const unsigned int CICHistory::DCWID = (unsigned int)-1;

CICHistory::~CICHistory()
{
}

bool CICHistory::seenBefore(unsigned int wid)
{
    return false;
}

bool CICHistory::memorize(unsigned int* its_wid, unsigned int* ite_wid)
{
    return true;
}

double CICHistory::pr(unsigned int* its_wid, unsigned int* ite_wid)
{
    return 0.0;
}

double CICHistory::pr(unsigned int* its_wid, unsigned int* ite_wid, unsigned int wid)
{
    return 0.0;
}

bool CICHistory::bufferize(void** buf_ptr, size_t* sz)
{
    *buf_ptr = NULL;
    *sz = 0;
    return true;
}

bool CICHistory::loadFromBuffer(void* buf_ptr, size_t sz)
{
    return true;
}


static bool bBigramHistoryInited = false;
const size_t  CBigramHistory::contxt_memory_size = 8192;
std::set<unsigned int>  CBigramHistory::s_stopWords;

/**
* Adding stop words including :
* 的 地 得 着 了 你 我 他 她 它 你们 我们 他们
* 们
* ??是 使 将
*/

void CBigramHistory::initClass()
{
    if (bBigramHistoryInited == false) {
        bBigramHistoryInited = true;
        s_stopWords.clear();

        s_stopWords.insert(0);     //unknown world
        s_stopWords.insert(DCWID); //seperator word id used by history memory interanlly

        s_stopWords.insert(67659); //的
        s_stopWords.insert(24261); //地
        s_stopWords.insert(37471); //得
        s_stopWords.insert(68920); //着
        s_stopWords.insert(5071);  //了

        s_stopWords.insert(8396);  //你
        s_stopWords.insert(40646); //我
        s_stopWords.insert(6755);  //他
        s_stopWords.insert(28369); //她
        s_stopWords.insert(30143); //它
        s_stopWords.insert(8398);  //你们
        s_stopWords.insert(40648); //我们
        s_stopWords.insert(6759);  //他们
        s_stopWords.insert(28370); //她们
        s_stopWords.insert(30114); //它们

        s_stopWords.insert(7121);  //们
    }
}

//FIXME: CBigramHistory need to be thread safe
CBigramHistory::CBigramHistory() : m_memory(), m_unifreq(), m_bifreq()
{

}

CBigramHistory::~CBigramHistory()
{
}

bool CBigramHistory::memorize(unsigned int* its_wid, unsigned int* ite_wid)
{
    TBigram bigram(DCWID, DCWID);

    // First , we insert an DC word id before the context history
    // to seperated from previous stream.
    if (m_memory.size() == contxt_memory_size) {
        TBigram hb;
        hb.first = m_memory.front();
        m_memory.pop_front();
        hb.second = m_memory.front();

        decUniFreq(hb.first);
        decBiFreq(hb);
    }
    m_memory.push_back(DCWID);

    //Now trying to memorize new stream and forget oldest
    for (; its_wid != ite_wid; ++its_wid) {
        if (m_memory.size() == contxt_memory_size) {
            TBigram hb;
            hb.first = m_memory.front();
            m_memory.pop_front();
            hb.second = m_memory.front();

            decUniFreq(hb.first);
            decBiFreq(hb);
        }
        bigram.first = bigram.second;
        bigram.second = *its_wid;
        m_memory.push_back(*its_wid);
        incUniFreq(bigram.second);
        incBiFreq(bigram);
    }
    return true;
}

double CBigramHistory::pr(unsigned int* its_wid, unsigned int* ite_wid)
{
    TBigram bigram(DCWID, DCWID);
    if (its_wid != ite_wid) {
        --ite_wid;
        bigram.second = *ite_wid;
        if (its_wid != ite_wid)
            bigram.first = *(ite_wid-1);
    }
    return pr(bigram);
}

double CBigramHistory::pr(unsigned int* its_wid, unsigned int* ite_wid, unsigned int wid)
{
    TBigram bigram(DCWID, DCWID);
    if (its_wid != ite_wid)
        bigram.first = *(ite_wid-1);
    bigram.second = wid;
    return pr(bigram);
}

static unsigned int s_word_endian = 0x1;

static bool isBigEndian(void)
{
   return ((*((char *)(&s_word_endian))) == 0);
}

inline unsigned short swap16(unsigned short x)
{
    return ((x << 8) | ((x >> 8) & 0xff));
}

inline unsigned swap32(unsigned x)
{
    return ((swap16(x) << 16) | (swap16(x >> 16) & 0xffff));
}

bool CBigramHistory::bufferize(void** buf_ptr, size_t* sz)
{
    bool bBigEndian = isBigEndian();
    *buf_ptr = NULL;
    *sz = 0;
    try {
        *sz = sizeof(unsigned int) * m_memory.size();
        if (*sz > 0) {
            *buf_ptr = malloc(*sz); // malloc for C compatible
            TContextMemory::const_iterator its = m_memory.begin();
            TContextMemory::const_iterator ite = m_memory.end();
            for (unsigned int* pw = (unsigned int*)*buf_ptr; its != ite; ++its) {
                *pw++ = (bBigEndian)?(*its):(swap32(*its));
            }
        }
        return true;
    } catch (...) {
        if (*buf_ptr)
            free(*buf_ptr);
        *buf_ptr = NULL;
        *sz = 0;
    }
    return false;
}

bool CBigramHistory::loadFromBuffer(void* buf_ptr, size_t sz)
{
    m_memory.clear();
    m_unifreq.clear();
    m_bifreq.clear();

    sz /= sizeof(unsigned int);
    unsigned int *pw = (unsigned int *)buf_ptr;
    bool bBigEndian = isBigEndian();

    if (pw && sz > 0) {
        if (!bBigEndian) {
            for (int i=0; i < sz; ++i) {
                *pw = swap32(*pw);
                ++pw;
            }
            pw = (unsigned int *)buf_ptr;
        }
        TBigram bigram(DCWID, DCWID);
        for (int i=0; i < sz; ++i) {
            bigram.first = bigram.second;
            bigram.second = *pw++;
            m_memory.push_back(bigram.second);
            incUniFreq(bigram.second);
            incBiFreq(bigram);
        }
    }
    return true;
}

double CBigramHistory::pr(TBigram& bigram)
{
    int uf0 = uniFreq(bigram.first);
    int bf = biFreq(bigram);
    int uf1 = uniFreq(bigram.second);
    double pr = 0.0;
    pr += 0.68*double(bf)/double(uf0+0.5);
    pr += 0.32*double(uf1)/double(m_memory.size() + (contxt_memory_size-m_memory.size())/10);
    //if (pr != 0) printf("cache pr(%d|%d) = %lf\n", bigram.second, bigram.first, pr);
    return pr;
}

int  CBigramHistory::uniFreq(TUnigram& ug)
{
    int freq = 0;
    if (s_stopWords.find(ug) == s_stopWords.end()) {
        TUnigramPool::iterator it = m_unifreq.find(ug);
        if (it != m_unifreq.end()) {
            freq = it->second;
        }
    }
    //if (freq != 0) printf("uniFreq[%d]-->%d\n", ug, freq);
    return freq;
}

int  CBigramHistory::biFreq(TBigram& bg)
{
    int freq = 0;
    //std::set<unsigned int>::const_iterator ite = s_stopWords.end();
    if (bg.first != DCWID && bg.second != DCWID) {
        TBigramPool::const_iterator it = m_bifreq.find(bg);
        if (it != m_bifreq.end())
            freq =  it->second;
    }

    //if (freq != 0) printf("biFreq[%d,%d]-->%d\n", bg.first, bg.second, freq);
    return freq;
}

void CBigramHistory::decUniFreq(TUnigram& ug)
{
    TUnigramPool::iterator it = m_unifreq.find(ug);
    if (it != m_unifreq.end()) {
        if (it->second > 1)
            --(it->second);
        else
            m_unifreq.erase(it);
    }
}

bool CBigramHistory::seenBefore(unsigned int wid)
{
    return (wid != DCWID && s_stopWords.find(wid) == s_stopWords.end() &&
            m_unifreq.find(wid) != m_unifreq.end());
}

void CBigramHistory::decBiFreq(TBigram& bg)
{
    TBigramPool::iterator it = m_bifreq.find(bg);
    if (it != m_bifreq.end()) {
        if (it->second > 1)
            --(it->second);
        else
            m_bifreq.erase(it);
    }
}

void CBigramHistory::incUniFreq(TUnigram& ug)
{
    ++m_unifreq[ug];
    //printf("Remebering uniFreq[%d]-->%d\n", ug, m_unifreq[ug]);
}

void CBigramHistory::incBiFreq(TBigram& bg)
{
    ++m_bifreq[bg];
    //printf("Remebering biFreq[%d,%d]-->%d\n", bg.first, bg.second, m_bifreq[bg]);
}
