/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").  
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at sunpinyin/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at sunpinyin/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#ifndef _SUNPINYIN_CONTEXT_HISTORY_H
#define _SUNPINYIN_CONTEXT_HISTORY_H

#include "portability.h"

#include <map>
#include <deque>
#include <set>

/**
* A forget all history memory
*/
class CICHistory {
public:
    /** don't care word id, or seperator word id */
    static const unsigned int DCWID;

    virtual ~CICHistory();

    virtual bool seenBefore(unsigned int wid);

    /**
    * memorize the context stream pointed by [its_wid, ite_wid)
    */
    virtual bool memorize(unsigned int* its_wid, unsigned int* ite_wid);

    /**
    * @param its_wid is the first word pointer of the context stream
    * @param ite_wid is the last (exclusive) word pointer of the context stream
    * @return pr(*(ite_wid-1) | *its_wid, ..., *(ite_wid-2))
    * The return value could be zero, i.e. no need to smooth the probabilities
    */
    virtual double pr(unsigned int* its_wid, unsigned int* ite_wid);

    /**
    * @param its_wid is the first word pointer of the history stream
    * @param ite_wid is the last (exclusive) word pointer of the history stream
    * @return pr(*wid | *its_wid, ..., *(ite_wid-1))
    * The return value could be zero, i.e. no need to smooth the probabilities
    */
    virtual double pr(unsigned int* its_wid, unsigned int* ite_wid, unsigned int wid);

    /**
    * allocate a buffer, and put the context memory's contect into it
    * @param buf_ptr would be stored the buffer pointer
    * @param sz would be the size in byte of the buffer allocated
    * @return false on error
    * Note: the buf_ptr should be used free(*buf_ptr) to free after usage
    */
    virtual bool
    bufferize(void** buf_ptr, size_t* sz);

    /**
    * Load context memory according to the buf
    * @param buf_ptr uffer pointer
    * @param sz is the size in byte of the buffer
    * @return false on error
    * call with buf_ptr with NULL value would clear the context memory
    */
    virtual bool
    loadFromBuffer(void* buf_ptr, size_t sz);
};

class CBigramHistory : public CICHistory {
public:
    static void initClass();

    CBigramHistory();

    virtual ~CBigramHistory();

    virtual bool seenBefore(unsigned int wid);

    virtual bool memorize(unsigned int* its_wid, unsigned int* ite_wid);

    /**
    * @param its_wid is the first word pointer of the context stream
    * @param ite_wid is the last (exclusive) word pointer of the context stream
    * @return pr(*(ite_wid-1) | *(ite_wid-2))
    */
    virtual double pr(unsigned int* its_wid, unsigned int* ite_wid);

    /**
    * @param its_wid is the first word pointer of the history stream
    * @param ite_wid is the last (exclusive) word pointer of the history stream
    * @return pr(*wid | *(ite_wid-1))
    */
    virtual double pr(unsigned int* its_wid, unsigned int* ite_wid, unsigned int wid);

    virtual bool
    bufferize(void** buf_ptr, size_t* sz);

    virtual bool
    loadFromBuffer(void* buf_ptr, size_t sz);

protected:
    typedef unsigned                              TWordId;
    typedef std::pair<TWordId, TWordId>           TBigram;
    typedef TWordId                               TUnigram;
    typedef std::map<TBigram, int>                TBigramPool;
    typedef std::map<TUnigram, int>               TUnigramPool;
    typedef std::deque<TWordId>                   TContextMemory;

    static const size_t contxt_memory_size;

    TContextMemory          m_memory;
    TUnigramPool            m_unifreq;
    TBigramPool             m_bifreq;

    static std::set<unsigned int>                  s_stopWords;

protected:
    double pr(TBigram& bg);
    int  uniFreq(TUnigram& ug);
    int  biFreq(TBigram& bg);

    void decUniFreq(TUnigram& ug);
    void decBiFreq(TBigram& bg);
    void incUniFreq(TUnigram& ug);
    void incBiFreq(TBigram& bg);
};

#endif
