/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").  
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at sunpinyin/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at sunpinyin/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iconv.h>

#include "imi_data.h"

void
print_wide(const TWCHAR* wstr)
{
    char buf[512];

    iconv_t icv = iconv_open("UTF-8", TWCHAR_ICONV_NAME);
    TIConvSrcPtr src = (TIConvSrcPtr)wstr;
    size_t srclen = (WCSLEN(wstr)+1)*sizeof(TWCHAR);
    char *dst = buf;
    size_t dstlen = 1024;
    iconv(icv, &src, &srclen, &dst, &dstlen);
    iconv_close(icv);
    printf("%s", buf);
}

bool
CIMIData::loadResource(const char* lm_file_path, const char* pytrie_file_path)
{
    clear();

    #ifdef DEBUG
        printf("\n    openning lm file %s, pytrie file %s...", lm_file_path, pytrie_file_path);
    #endif

    if ((m_pLM = new CThreadSlm()) != NULL && m_pLM->load(lm_file_path, true) &&
        (m_pPinyinTrie = new CPinyinTrie()) != NULL && m_pPinyinTrie->load(pytrie_file_path)) {

        #ifdef DEBUG
            printf("done! LM @0x%X, Trie @0x%X\n", (int)m_pLM, (int)m_pPinyinTrie);
            fflush(stdout);
        #endif

        return true;
    }

    clear();

    #ifdef DEBUG
        printf("fail!\n");
        fflush(stdout);
    #endif

    return false;
}

void
CIMIData::clear()
{
    if (m_pLM)
        delete m_pLM;

    if (m_pPinyinTrie)
        delete m_pPinyinTrie;

    m_pPinyinTrie = NULL;
    m_pLM = NULL;
}
