/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").  
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at sunpinyin/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at sunpinyin/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "imi_uiobjects.h"

#include <algorithm>

#define MAX_CANDI_COUNT_PER_WINDOW      16
#define MAX_STRING_LEN_CANDI_PREEDIT    256

IPreeditString::~IPreeditString()
{
}

CPreEditString::CPreEditString() 
    : m_caret(0), m_candi_start(0), m_wstr(), m_charTypes()
{
    m_charTypes.reserve(MAX_STRING_LEN_CANDI_PREEDIT);
}

CPreEditString::~CPreEditString()
{
}

int
CPreEditString::size() const
{
    return m_wstr.size();
}

const TWCHAR*
CPreEditString::string() const
{
    return m_wstr.c_str();
}

int
CPreEditString::charTypeSize() const
{
    return std::min(m_charTypes.size(), m_wstr.size());
}

int
CPreEditString::charTypeAt(int idx) const
{
    return (idx < 0 || idx >= m_charTypes.size())?(DONTCARE_CHAR):(m_charTypes[idx]);
}

int
CPreEditString::caret() const
{
    return m_caret;
}

int
CPreEditString::candi_start() const
{
    return m_candi_start;
}

void
CPreEditString::clear()
{
    m_caret = 0;
    m_wstr.clear();
    m_charTypes.clear();
}

wstring&
CPreEditString::getString()
{
    return m_wstr;
}

IPreeditString::CCharTypeVec &
CPreEditString::getCharTypeVec()
{
    return m_charTypes;
}

void
CPreEditString::setCaret(int caret)
{
    m_caret = caret;
}

void
CPreEditString::setCandiStart(int candi_start)
{
    m_candi_start = candi_start;
}

ICandidateList::~ICandidateList()
{
}

CCandidateList::CCandidateList() 
    : m_candiStrings(), m_candiTypes(), m_candiCharTypeVecs(),
      m_first(0), m_total(0)
{
}

CCandidateList::~CCandidateList()
{
}

int
CCandidateList::size() const
{
    return m_candiStrings.size();
}

int
CCandidateList::total() const
{
    return m_total;
}

int
CCandidateList::first() const
{
    return m_first;
}

void
CCandidateList::setTotal(int total)
{
    m_total = total;
}

void
CCandidateList::setFirst(int first)
{
    m_first = first;
}

int
CCandidateList::candiType(unsigned int item) const
{
    return (item < 0 || item >= m_candiTypes.size())?(NORMAL_WORD):(m_candiTypes[item]);
}

int
CCandidateList::candiSize(unsigned int item) const
{
    return (item < 0 || item >= m_candiStrings.size())?(0):(m_candiStrings[item].size());
}

const TWCHAR*
CCandidateList::candiString(unsigned int item) const
{
    return (item < 0 || item >= m_candiStrings.size())?(NULL):(m_candiStrings[item].c_str());
}

int
CCandidateList::candiCharTypeSizeAt(unsigned int item) const
{
    if (item < 0 || item >= m_candiCharTypeVecs.size())
        return 0;
    const CCharTypeVec & ctv = m_candiCharTypeVecs[item];
    return std::min((int)ctv.size(), candiSize(item));
}

int
CCandidateList::candiCharTypeAt(unsigned int item, unsigned int idx) const
{
    int sz = candiCharTypeSizeAt(item);
    return (idx < 0 || idx >= sz)?(DONTCARE_CHAR):(m_candiCharTypeVecs[item][idx]);
}

void
CCandidateList::clear()
{
    m_first = m_total = 0;
    m_candiStrings.clear();
    m_candiTypes.clear();
    m_candiCharTypeVecs.clear();
}

void
CCandidateList::reserve(int count)
{
    m_candiStrings.reserve(count);
    m_candiTypes.reserve(count);
    m_candiCharTypeVecs.reserve(count);
}

ICandidateList::CCandiStrings &
CCandidateList::getCandiStrings()
{
    return m_candiStrings;
}

ICandidateList::CCandiTypeVec &
CCandidateList::getCandiTypeVec()
{
    return m_candiTypes;
}

ICandidateList::CCharTypeVecs &
CCandidateList::getCharTypeVecs()
{
    return m_candiCharTypeVecs;
}
