/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").  
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at sunpinyin/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at sunpinyin/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#ifndef SUNPINYIN_UI_OBJECTS_H
#define SUNPINYIN_UI_OBJECTS_H

#include "portability.h"

#include <vector>
#include <string>

#include "imi_context.h"

class IECharType {
public:
    enum ECharType {
        DONTCARE_CHAR   = 0,

        NORMAL_CHAR     = 1,
        PINYIN_CHAR     = 2 + NORMAL_CHAR,
        ASCII_CHAR      = ((PINYIN_CHAR - NORMAL_CHAR) << 1) + NORMAL_CHAR,
        SIMBOL_CHAR     = ((ASCII_CHAR - NORMAL_CHAR) << 1) + NORMAL_CHAR,
        PUNC_CHAR       = ((SIMBOL_CHAR - NORMAL_CHAR) << 1) + NORMAL_CHAR,
        HANZI_CHAR      = ((PUNC_CHAR - NORMAL_CHAR) << 1) + NORMAL_CHAR,

        BOUNDARY        = 256,

        CONVERTED       = 0x10000,
        USER_CHOICE     = (CONVERTED << 1),
        ILLEGAL         = (USER_CHOICE << 1),
        SELECTION       = (ILLEGAL << 1)
    };
};

/** The interface for retrieve Preedit data*/
class IPreeditString : public virtual IECharType {
public:
    typedef std::vector<int>    CCharTypeVec;

public:
    virtual
    ~IPreeditString()                                       = 0;

    /*@{*/
    /** Following functions are used for CIMIWinHandler to retrieve Preedit data. */
    virtual int                 size() const                        = 0;
    virtual const TWCHAR*       string() const                      = 0;
    virtual int                 charTypeSize() const                = 0;
    virtual int                 charTypeAt(int idx) const           = 0;
    virtual int                 caret() const                       = 0;
    virtual int                 candi_start() const                 = 0;
    /*@}*/

public:
    /*@{*/
    /**
    * Following functions are used for CIMIView to set Preedit data, should not
    * be used by CIMIWinHandler.
    */
    virtual void                clear()             = 0;
    virtual wstring&            getString()         = 0;
    virtual CCharTypeVec&       getCharTypeVec()    = 0;
    virtual void                setCaret(int caret) = 0;
    virtual void                setCandiStart(int s)= 0;
    /*@}*/
};

class CPreEditString : virtual public IPreeditString {
public:
    CPreEditString();

    virtual
    ~CPreEditString();

    /*@{*/
    virtual int                 size() const;
    virtual const TWCHAR*       string() const;
    virtual int                 charTypeSize() const;
    virtual int                 charTypeAt(int idx) const;
    virtual int                 caret() const;
    virtual int                 candi_start() const;
    /*@}*/

    /*@{*/
    virtual void                clear();
    virtual wstring&            getString();
    virtual CCharTypeVec&       getCharTypeVec();
    virtual void                setCaret(int caret);
    virtual void                setCandiStart(int s);
    /*@}*/

protected:
    wstring             m_wstr;
    int                 m_caret;
    int                 m_candi_start;
    CCharTypeVec        m_charTypes;
};


/** The interface for retrieve Preedit candidate data*/
class ICandidateList : public virtual IECharType {
public:
    enum ECandidateType {
        NORMAL_WORD = 0,
        AS_INPUT,
        BEST_WORD,
        USER_SELECTED_WORD,
        BEST_TAIL,
        OTHER_BEST_TAIL
    };

    typedef IPreeditString::CCharTypeVec        CCharTypeVec;
    typedef std::vector< CCharTypeVec >         CCharTypeVecs;
    typedef std::vector< wstring >              CCandiStrings;
    typedef std::vector<int>                    CCandiTypeVec;

public:
    virtual
    ~ICandidateList()                                           = 0;

    /*@{*/
    /** Following functions are used for CIMIWinHandler to retrieve Candidate data. */
    virtual int                 size() const                            = 0;
    virtual int                 total() const                           = 0;
    virtual int                 first() const                           = 0;

    virtual int                 candiType(unsigned int item) const      = 0;
    virtual int                 candiSize(unsigned int item) const      = 0;
    virtual const TWCHAR*       candiString(unsigned int item) const    = 0;

    virtual int                 candiCharTypeSizeAt(unsigned int item) const                = 0;
    virtual int                 candiCharTypeAt(unsigned int item, unsigned int idx) const  = 0;
    /*@}*/

public:
    /*@{*/
    /**
    * Following functions are used for CIMIView to set Candidate List data, should not
    * be used by CIMIWinHandler.
    */
    virtual void                clear()                    = 0;
    virtual void                setTotal(int total)        = 0;
    virtual void                setFirst(int first)        = 0;

    virtual void                reserve(int count)         = 0;
    virtual CCandiStrings &     getCandiStrings()          = 0;
    virtual CCandiTypeVec &     getCandiTypeVec()          = 0;
    virtual CCharTypeVecs &     getCharTypeVecs()          = 0;
    /*@}*/
};

class CCandidateList : virtual public ICandidateList {
public:
    CCandidateList();

    virtual
    ~CCandidateList();

    /*@{*/
    virtual int                 size() const;
    virtual int                 total() const;
    virtual int                 first() const;

    virtual int                 candiType(unsigned int item) const;
    virtual int                 candiSize(unsigned int item) const;
    virtual const TWCHAR*       candiString(unsigned int item) const;

    virtual int                 candiCharTypeSizeAt(unsigned int item) const;
    virtual int                 candiCharTypeAt(unsigned int item, unsigned int idx) const;
    /*@}*/

    /*@{*/
    virtual void                clear();
    virtual void                setTotal(int total);
    virtual void                setFirst(int first);
    virtual void                reserve(int count);
    virtual CCandiStrings &     getCandiStrings();
    virtual CCandiTypeVec &     getCandiTypeVec();
    virtual CCharTypeVecs &     getCharTypeVecs();
    /*@}*/

protected:
    int                 m_total;
    int                 m_first;
    CCandiStrings       m_candiStrings;
    CCandiTypeVec       m_candiTypes;
    CCharTypeVecs       m_candiCharTypeVecs;

};

/**
struct TSimplePreEditData {
    wstring             m_wstr;
    int                 m_caret;
};

struct TDecoratedPreEditData : public TSimplePreEditData {

    enum {
        DONTCARE_CHAR = 0,
        PINYIN_CHAR = 1,
        ASCII_CHAR = 2,
        SIMBOL_CHAR = 4,
        PUNC_CHAR = 8,
        AUTO_BOUND = 16,
        USER_BOUND = 32,
        CONVERTED_CHAR = 64,
        SELECTED_CHAR = 128,
        ERRORED_CHAR = 256
    };

    //int                 m_SyllPos;
    //int                 m_SyllSize;
    std::vector<int>    m_charTypes;

    inline int
    getCharTypeAtIdx(int idx)
        {
            if (m_charTypes.size() <= idx)
                return DONTCARE_CHAR;
            else
                return m_charTypes[idx];
        }
};

struct TSimpleCandidatesData {
    enum {
        NORMAL_WORD = 0,
        BEST_WORD,
        BEST_USER_SELECTED,
        COMPOSITE_BEST_TAIL
    };
    wstring             m_bestTail;
    CCandidates        *mp_candidates;
    int                 m_first;
    int                 m_winSize;
public:
    //get the candidate string inside current candidate area at idx (starting from 0)
    inline const TWCHAR*
    getItemInWindow(int idx)
        {
            idx += m_first;
            if (m_bestTail.size() > 0)
                --idx;
            if (idx < 0)
                return m_bestTail.c_str();
            else if (idx >= mp_candidates->size())
                return NULL;
            else
                return mp_candidates->operator[](idx).m_String;
        }

    inline int
    getItemTypeInWindow(int idx)
        {
            idx += m_first;
            if (m_bestTail.size() > 0)
                --idx;
            if (idx < 0)
                return COMPOSITE_BEST_TAIL;
            else if (idx == 0)
                return BEST_WORD;
            else
                return NORMAL_WORD;
        }
};
*/

#endif
