/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").  
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at sunpinyin/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at sunpinyin/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#ifndef SUNPY_IMI_SESSION_VIEW_H
#define SUNPY_IMI_SESSION_VIEW_H

#include "portability.h"

#include "imi_options.h"

#include "imi_context.h"
#include "imi_winHandler.h"
#include "imi_uiobjects.h"

class CIMIView;

class CIMIViewFactory {
    static int          sm_mapInit;
public:
    enum {
        SVT_NONE = 0,
        SVT_MODERN,
        SVT_CLASSIC
    };

    static CIMIView *
    createView(int viewType);
};

class CIMIView {
public:
    enum {
        KEYEVENT_USED =  (1),
        PREEDIT_MASK =   (1<<2),
        CANDIDATE_MASK = (1<<3)
    };

public:
    static int s_CandiWindowSize;

public:
    CIMIView();

    /**{*/
    CIMIOptions* getPreference(void)
        { return m_pPref; }

    void attachWinHandler(CIMIWinHandler* wh)
            { mp_winHandler = wh; }

    CIMIWinHandler* getWinHandler(void)
            { return mp_winHandler; }

    CIMIContext* getIC(void)
            { return m_pIC; }
    /**}*/

    /*@{*/
    virtual ~CIMIView();

    virtual int  getViewType(void);

    virtual void setPreference(const CIMIOptions* pp);

    virtual void attachIC(CIMIContext* pIC);

    virtual unsigned clearIC(void);

    virtual void updateWindows(unsigned int mask);

    virtual void setStatusAttrValue(int key, int value);

    virtual int  onKeyEvent(unsigned keycode, unsigned keyvalue, unsigned modifier);

    virtual int  onCandidatePageRequest(int pgno, bool relative); //pgno == -1, relative == false means last page

    virtual int  onCandidateSelectRequest(int index);

    virtual int  getStatusAttrValue(int key);
    /*@}*/

    /* Functions to retrieve the Preedit/Candidate list */
    /*@{*/
    virtual void getInputString(wstring& s);

    virtual void getPreeditString(IPreeditString& ps);

    virtual void getCandidateList(ICandidateList& cl, int start, int size);
    /*@}*/

protected:
    /*@{*/
    /** Punc include . , ; : ? " ... */
    static bool
    isPuncChar(TWCHAR key);

    /** simbol is those not punc, but has full wide mode */
    static bool
    isSimbolChar(TWCHAR key);

    /** Whether or not the wide char wc is a end-of-senetence puncuation */
    static bool
    isTermPuncWide(TWCHAR wc);
    /*@}*/

    /*@{*/
    /** Map the half Punc char into its full wide char*/
    TWCHAR
    getFullPunc(TWCHAR key);

    /** Map the half simbol char into its full wide char*/
    TWCHAR
    getFullSimbol(TWCHAR key);
    /*@}*/

protected:
    /*@{*/
    CIMIContext        *m_pIC;
    CIMIWinHandler*     mp_winHandler;
    /*@}*/

    /*@{*/
    bool                m_CN;
    bool                m_FullPunc;
    bool                m_FullSimbol;
    bool                m_bRightFullDblQuote;

    CIMIOptions*        m_pPref;
    /*@}*/
};

#endif
