/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").  
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at sunpinyin/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at sunpinyin/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "imi_winHandler.h"
#include "imi_view.h"

CIMIWinHandler::~CIMIWinHandler()
{
}

void
CIMIWinHandler::setOptions(CIMIOptions* pp)
{
    m_pPrefLink = pp;
}

void
CIMIWinHandler::commit(const TWCHAR* wstr)
{
    if (wstr) print_wide(wstr);
    fflush(stdout);
}

void
CIMIWinHandler::updatePreedit(const IPreeditString* ppd)
{
    if (ppd) {
        print_wide(ppd->string());
        printf("\n");
        fflush(stdout);
    }
}

void
CIMIWinHandler::updateCandidates(const ICandidateList* pcl)
{
    for (int i=0, sz=pcl->size(); i < sz; ++i) {
        const TWCHAR* pcand = pcl->candiString(i);
        if (pcand != NULL) {
            printf("%c. ", '1' + i);
            print_wide(pcand);
            printf("\n");
        }
    }
    fflush(stdout);
}

void
CIMIWinHandler::throwBackKey(unsigned keycode, unsigned keyvalue, unsigned modifier)
{
    if (keyvalue > 0x0 && keyvalue < 0x7f) {
        printf("%c", keyvalue);
        fflush(stdout);
    }
}

void
CIMIWinHandler::updateStatus(int key, int value)
{
    switch (key) {
    case STATUS_ID_CN:
        printf("CN status is "); break;
    case STATUS_ID_FULLPUNC:
        printf("Full Punc is "); break;
    case STATUS_ID_FULLSIMBOL:
        printf("Full Simbol is "); break;
    default:
        printf("%^&*()Unknow Status id %d is ", key); break;
    }

    printf("%d\n", value);
    fflush(stdout);
}
