/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").  
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at sunpinyin/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at sunpinyin/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "portability.h"

#include <iconv.h>

/**
* convert UTF-8 string pointed by s into UCS-4 Wide String at pwcs.
* No more than n wide char could be converted into target buffer.
* return -1 means error.
* other means the converted wide char number do not count the end 0.
*/
size_t MBSTOWCS(TWCHAR *pwcs, const char* s, size_t n)
{
    static iconv_t ic = iconv_open(TWCHAR_ICONV_NAME, "UTF-8");

    // To eliminate the const char* and char* diffirence in differnt system
    TIConvSrcPtr src = (TIConvSrcPtr)s;
    size_t srclen = strlen(s)+1;
    char* dst = (char *)pwcs;
    size_t dstlen = n*sizeof(TWCHAR);

    size_t res = iconv(ic, &src, &srclen, &dst, &dstlen);

    if (res != size_t(-1) && srclen == 0) {
        n -= dstlen/sizeof(TWCHAR);
        return (n > 0)?(n-1):0;
    } else {
        return size_t(-1);
    }
}

/**
* convert UCS-4 string pointed by pwcs into UTF-8 String at s.
* No more than n byte could be converted into target buffer.
* return -1 means error.
* Other means the converted byte number do not count the end 0.
*/
size_t WCSTOMBS(char* s, const TWCHAR* pwcs, size_t n)
{
    static iconv_t ic = iconv_open("UTF-8", TWCHAR_ICONV_NAME);

    TIConvSrcPtr src = (TIConvSrcPtr)pwcs;
    size_t srclen = (WCSLEN(pwcs)+1)*sizeof(TWCHAR);
    char* dst = (char *)s;
    size_t dstlen = n;

    size_t res = iconv(ic, &src, &srclen, &dst, &dstlen);

    if (res != size_t(-1) && srclen == 0) {
        n -= dstlen;
        return (n > 0)?(n-1):0;
    } else {
        return size_t(-1);
    }
}

/**
* return the wide string len at pwcs, not count the end 0.
*/
size_t WCSLEN(const TWCHAR* pwcs)
{
    size_t sz = 0;
    if (pwcs) {
        while (*pwcs++)
            ++sz;
    }
    return sz;
}
