/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").  
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at sunpinyin/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at sunpinyin/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
#ifndef ___SUN_SLM_PORTABILITY_H___
#define ___SUN_SLM_PORTABILITY_H___

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "host_os.h"
#include "host_cpu.h"

#include <string>

#if defined(DEBUG) && !defined(NDEBUG)
    #define DEBUG_print(fmt, ...)   fprintf(stderr, fmt, ...)
#else
    #define DEBUG_print(fmt, ...)   (int(0))
#endif

/*
* UCS4 wide character type, system dependent
* Multibytes string in this program is UTF-8 only
*/
typedef unsigned         TWCHAR;

#if defined(MY_LITTLE_ENDIAN) && !defined(MY_BIG_ENDIAN)
    #define  TWCHAR_ICONV_NAME  "UCS-4LE"
#elif defined(MY_BIG_ENDIAN) && !defined(MY_LITTLE_ENDIAN)
    #define  TWCHAR_ICONV_NAME  "UCS-4BE"
#else
    #error "Please define one and only one of BIG_ENDIAN or LITTLE_ENDIAN !!!"
#endif

typedef TWCHAR           TSIMWordId;

#define  SIM_ID_NOT_CHAR (0xFFFF)

class TSIMChar {
public:
    inline TSIMChar() : m_ch(0)
        { }

    inline TSIMChar(const TSIMChar& r) : m_ch(r.m_ch)
        { }

    inline TSIMChar(unsigned char c) : m_ch(c)
        { }

    inline TSIMChar(const TWCHAR & wch)
        {
            m_ch = (wch > SIM_ID_NOT_CHAR)?(SIM_ID_NOT_CHAR):(wch);
        }

    //inline TSIMChar(unsigned int ui) same as TSIMChar(TWCHAR)

    //inline operator TWCHAR(void) const  same as follow

    inline operator unsigned int(void ) const
        { return m_ch; }

    inline operator bool() const
        { return m_ch != 0; }

    inline TSIMChar& operator= (const TSIMChar & r)
        { m_ch = r.m_ch; return *this; }

    inline bool operator == (const TSIMChar & r) const
        { return m_ch == r.m_ch; }

    inline bool operator != (const TSIMChar & r) const
        { return m_ch != r.m_ch; }

    inline bool operator <  (const TSIMChar & r) const
        { return m_ch <  r.m_ch; }

    inline bool operator >  (const TSIMChar & r) const
        { return m_ch >  r.m_ch; }

    inline bool operator <= (const TSIMChar & r) const
        { return m_ch <= r.m_ch; }

    inline bool operator >= (const TSIMChar & r) const
        { return m_ch >= r.m_ch; }

private:
    unsigned short m_ch;
};

#if defined(MY_LITTLE_ENDIAN) && !defined(MY_BIG_ENDIAN)


#elif defined(MY_BIG_ENDIAN) && !defined(MY_LITTLE_ENDIAN)

#else
#error "Please define one and only one of BIG_ENDIAN or LITTLE_ENDIAN !!!"

#endif

const TSIMWordId SIM_ID_NOT_WORD        = (0x0);
const TSIMWordId SIM_ID_UNKNOWN_CN      = (2);
const TSIMWordId SIM_ID_SEN_TOKEN       = (10);
const TSIMWordId SIM_ID_DIGIT           = (20);
const TSIMWordId SIM_ID_SIMBOL          = (21);
const TSIMWordId SIM_ID_DUMMY_WORD      = (69);
const TSIMWordId SIM_ID_NONWORD         = (69);
const TSIMWordId SIM_ID_ALLWORD         = (69);
const TSIMWordId SIM_ID_REALWORD_START  = (70);
const TSIMWordId SIM_ID_CNWORD_START    = (100);

const TWCHAR WCH_NULL          =  0;
const TWCHAR WCH_RETURN        =  '\n';
const TWCHAR WCH_TAB           =  '\t';
const TWCHAR WCH_SPACE         =  ' ';
const TWCHAR WCH_LESSTHAN      =  '<';
const TWCHAR WCH_GREATERTHAN   =  '>';

const TWCHAR WCH_DOUBLESPACE   =  0x3000;
const TWCHAR WCH_JUHAO         =  0x3002;
const TWCHAR WCH_WENHAO        =  0xFF1F;
const TWCHAR WCH_TANHAO        =  0xFF01;
const TWCHAR WCH_FENHAO        =  0xFF1B;
const TWCHAR WCH_MAOHAO        =  0xFF1A;
const TWCHAR WCH_DOUHAO        =  0xFF0C;
const TWCHAR WCH_ZUOKUOHAO     =  0x201C;
const TWCHAR WCH_YOUKUOHAO     =  0x201D;
const TWCHAR WCH_SHENGLUEHAO   =  0x2026;


size_t MBSTOWCS(TWCHAR *pwcs, const char* s, size_t n);

size_t WCSTOMBS(char* s, const TWCHAR* pwcs, size_t n);

size_t WCSLEN(const TWCHAR* ws);

namespace std {

template<>
struct char_traits<TWCHAR>
{
    typedef TWCHAR            char_type;
    typedef unsigned int      int_type;
    typedef streamoff         off_type;
    typedef wstreampos        pos_type;
    typedef mbstate_t         state_type;

    static void
    assign(char_type& __c1, const char_type& __c2)
    { __c1 = __c2; }

    static bool
    eq(const char_type& __c1, const char_type& __c2)
    { return __c1 == __c2; }

    static bool
    lt(const char_type& __c1, const char_type& __c2)
    { return __c1 < __c2; }

    static int
    compare(const char_type* __s1, const char_type* __s2, size_t __n)
    {
        for (size_t i=0; i < __n; ++i) {
            if (*__s1 < *__s2)
                return -1;
            else if (*__s1++ == *__s2++)
                continue;
            else
                return 1;
        }
        return 0;
    }

    static size_t
    length(const char_type* __s)
    { return WCSLEN(__s); }

    static const char_type*
    find(const char_type* __s, size_t __n, const char_type& __a)
    {
        while (__n--) {
            if (*__s++ == __a)
                return __s;
        }
        return NULL;
    }

    static char_type*
    move(char_type* __s1, const char_type* __s2, int_type __n)
    {
        return static_cast<char_type*>(memmove(__s1, __s2, __n*sizeof(char_type)));
    }

    static char_type*
    copy(char_type* __s1, const char_type* __s2, size_t __n)
    {
        return static_cast<char_type*>(memcpy(__s1, __s2, __n*sizeof(char_type)));
    }

    static char_type*
    assign(char_type* __s, size_t __n, char_type __a)
    {
        for (char_type*p = __s; __n--;)
            *p++ = __a;
        return __s;
    }

    static char_type
    to_char_type(const int_type& __c) { return char_type(__c); }

    static int_type
    to_int_type(const char_type& __c) { return int_type(__c); }

    static bool
    eq_int_type(const int_type& __c1, const int_type& __c2)
    { return __c1 == __c2; }

    static int_type
    eof() { return static_cast<int_type>(WEOF); }

    static int_type
    not_eof(const int_type& __c)
    { return eq_int_type(__c, eof()) ? 0 : __c; }
};

};

typedef std::basic_string<TWCHAR>   wstring;

#endif
