/*
 * Copyright (c) 2007 Kov Chai <tchaikov@gmail.com>
 */
#define Uses_STL_AUTOPTR
#define Uses_STL_FUNCTIONAL
#define Uses_STL_VECTOR
#define Uses_STL_IOSTREAM
#define Uses_STL_FSTREAM
#define Uses_STL_ALGORITHM
#define Uses_STL_MAP
#define Uses_STL_UTILITY
#define Uses_STL_IOMANIP
#define Uses_C_STDIO
#define Uses_SCIM_UTILITY
#define Uses_SCIM_IMENGINE
#define Uses_SCIM_ICONV
#define Uses_SCIM_CONFIG_BASE
#define Uses_SCIM_CONFIG_PATH
#define Uses_SCIM_LOOKUP_TABLE
#define Uses_SCIM_DEBUG

#include "portability.h"

#include <imi_scimwin.h>
#include <imi_winHandler.h>
#include <imi_uiobjects.h>

#include <scim.h>
#include "sunpinyin_utils.h"
#include "sunpinyin_lookup_table.h"
#include "sunpinyin_imengine.h"

using namespace scim;

CScimWinHandler::CScimWinHandler(SunPyInstance* ime, SunLookupTable* lookup_table)
    : CIMIWinHandler(), m_ime(ime), m_lookup_table(lookup_table)
{
    //
}

CScimWinHandler::~CScimWinHandler()
{
}

void
CScimWinHandler::setOptions(CIMIOptions* pp)
{
    m_pPrefLink = pp;
}

void
CScimWinHandler::commit(const TWCHAR* wstr)
{
    if (wstr) {
        SCIM_DEBUG_IMENGINE (3) << "commit(" << wstr << ")\n";
        m_ime->commit_string(wstr_to_widestr(wstr));
    }
}

void
CScimWinHandler::updatePreedit(const IPreeditString* ppd)
{
    if (ppd)
        m_ime->redraw_preedit_string(ppd);
}

void
CScimWinHandler::updateCandidates(const ICandidateList* pcl)
{
    if (pcl)
        m_ime->redraw_lookup_table(pcl);
}

void
CScimWinHandler::throwBackKey(unsigned keycode, unsigned keyvalue, unsigned modifier)
{
    if (keyvalue > 0x0 && keyvalue < 0x7f) {
        printf("%c", keyvalue);
        fflush(stdout);
    }
}

void
CScimWinHandler::updateStatus(int key, int value)
{
    switch (key) {
    case STATUS_ID_CN:
        m_ime->refresh_status_property(value != 0);
        break;
    case STATUS_ID_FULLPUNC:
        m_ime->refresh_fullpunc_property(value != 0);
        break;
    case STATUS_ID_FULLSIMBOL:
        m_ime->refresh_fullsimbol_property(value != 0);
        break;
    default:
        SCIM_DEBUG_IMENGINE (2) << "Should not happen: updateStatus(" << key << ", " << value << ")\n";
        break;
    }
}
