#define Uses_SCIM_CONFIG_BASE

#include <gtk/gtk.h>

#include <scim.h>
#include <gtk/scimkeyselection.h>

#include "sunpinyin_private.h"
#include "sunpinyin_imengine_config_keys.h"

using namespace scim;

#define scim_module_init sunpinyin_imengine_setup_LTX_scim_module_init
#define scim_module_exit sunpinyin_imengine_setup_LTX_scim_module_exit

#define scim_setup_module_create_ui       sunpinyin_imengine_setup_LTX_scim_setup_module_create_ui
#define scim_setup_module_get_category    sunpinyin_imengine_setup_LTX_scim_setup_module_get_category
#define scim_setup_module_get_name        sunpinyin_imengine_setup_LTX_scim_setup_module_get_name
#define scim_setup_module_get_description sunpinyin_imengine_setup_LTX_scim_setup_module_get_description
#define scim_setup_module_load_config     sunpinyin_imengine_setup_LTX_scim_setup_module_load_config
#define scim_setup_module_save_config     sunpinyin_imengine_setup_LTX_scim_setup_module_save_config
#define scim_setup_module_query_changed   sunpinyin_imengine_setup_LTX_scim_setup_module_query_changed

static GtkWidget * input_style_combo = NULL;
static GtkWidget * charset_combo = NULL;
static GtkWidget * minus_pageup_button = NULL;
static GtkWidget * bracket_pageup_button = NULL;
static GtkWidget * comma_pageup_button = NULL;


static GtkWidget * create_setup_window ();
static void        load_config (const ConfigPointer &config);
static void        save_config (const ConfigPointer &config);
static bool        query_changed ();

// Module Interface.
extern "C" {
    void scim_module_init (void)
    {
        bindtextdomain (GETTEXT_PACKAGE, SCIM_SUNPINYIN_LOCALEDIR);
        bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    }

    void scim_module_exit (void)
    {
    }

    GtkWidget * scim_setup_module_create_ui (void)
    {
        static GtkWidget *setup_ui = NULL;
        if (setup_ui == NULL)
            setup_ui = create_setup_window ();
        return setup_ui;
    }

    String scim_setup_module_get_category (void)
    {
        return String ("IMEngine");
    }

    String scim_setup_module_get_name (void)
    {
        return String (_("SunPinyin"));
    }

    String scim_setup_module_get_description (void)
    {
        return String (_("A Statistical Language Model Based Intelligent IMEngine Module for Simplified Chinese."));
    }

    void scim_setup_module_load_config (const ConfigPointer &config)
    {
        load_config (config);
    }

    void scim_setup_module_save_config (const ConfigPointer &config)
    {
        save_config (config);
    }

    bool scim_setup_module_query_changed ()
    {
        return query_changed ();
    }
} // extern "C"

static bool __have_changed                 = false;

static void
on_value_changed(GtkWidget *  widget,
                 gpointer     user_data)
{
        __have_changed = true;
}

static GtkWidget *
create_options_page(GtkTooltips *tooltips)
{
    GtkWidget *vbox;
    GtkWidget *label;
    GtkWidget *button;

    vbox = gtk_vbox_new (FALSE, 12);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);

    GtkWidget *table = gtk_table_new (2, 2, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, TRUE, 0);

    label = gtk_label_new (_("Input Style:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                      (GtkAttachOptions) (GTK_FILL),
                      (GtkAttachOptions) (GTK_FILL), 4, 4);


    GtkWidget *combo_box = gtk_combo_box_new_text();
    gtk_table_attach (GTK_TABLE (table), combo_box, 1, 2, 0, 1,
                      (GtkAttachOptions) (GTK_FILL),
                      (GtkAttachOptions) (GTK_FILL), 4, 4);
    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), _("Classic Style"));
    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), _("Microsoft Pinyin Style"));
    gtk_tooltips_set_tip(tooltips, combo_box, _("Changing this will dismiss the un-committed preedit strings"), NULL);
    g_signal_connect(G_OBJECT(combo_box), "changed",
                     G_CALLBACK(on_value_changed), NULL);
    input_style_combo = combo_box;

    label = gtk_label_new (_("Character Set:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                      (GtkAttachOptions) (GTK_FILL),
                      (GtkAttachOptions) (GTK_FILL), 4, 4);

    combo_box = gtk_combo_box_new_text();
    gtk_table_attach (GTK_TABLE (table), combo_box, 1, 2, 1, 2,
                      (GtkAttachOptions) (GTK_FILL),
                      (GtkAttachOptions) (GTK_FILL), 4, 4);
    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), _("GB2312"));
    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), _("GBK"));
    gtk_tooltips_set_tip(tooltips, combo_box, _("Changing this will dismiss the un-committed preedit strings"), NULL);
    g_signal_connect(G_OBJECT(combo_box), "changed",
                     G_CALLBACK(on_value_changed), NULL);
    charset_combo = combo_box;

    button = gtk_check_button_new_with_mnemonic (_("Use -/= for paging down/up"));
    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(button), "toggled",
                     G_CALLBACK(on_value_changed), NULL);
    minus_pageup_button = button;

    button = gtk_check_button_new_with_mnemonic (_("Use [/] for paging down/up"));
    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(button), "toggled",
                     G_CALLBACK(on_value_changed), NULL);
    bracket_pageup_button = button;

    button = gtk_check_button_new_with_mnemonic (_("Use ,/. for paging down/up"));
    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
    g_signal_connect(G_OBJECT(button), "toggled",
                     G_CALLBACK(on_value_changed), NULL);
    comma_pageup_button = button;

    return vbox;
}

static GtkWidget *
create_setup_window ()
{
    GtkWidget *notebook;
    GtkWidget *label;
    GtkWidget *page;
    GtkTooltips *tooltips;

    // Create the shared tooltips.
    tooltips = gtk_tooltips_new ();

    notebook = gtk_notebook_new ();

    // Create the option page. 
    page = create_options_page(tooltips);
    label = gtk_label_new (_("Options"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);

    gtk_notebook_set_current_page(GTK_NOTEBOOK (notebook), 0);

    gtk_widget_show_all(notebook);
    return notebook;
}

static void
load_config (const ConfigPointer &config)
{
    if (config.null())
        return;

    int viewtype = config->read (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_VIEW_TYPE), 0);
    gtk_combo_box_set_active (GTK_COMBO_BOX(input_style_combo), viewtype);

    int charset  = config->read (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_CHARHSET), 1);
    gtk_combo_box_set_active (GTK_COMBO_BOX(charset_combo), charset);

    bool stat;
    stat = config->read (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_PAGE_MINUS), true);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(minus_pageup_button), stat);

    stat = config->read (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_PAGE_BRACKET), true);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bracket_pageup_button), stat);

    stat = config->read (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_PAGE_COMMA), false);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(comma_pageup_button), stat);

    __have_changed = false;
}

static void
save_config (const ConfigPointer &config)
{
    if (config.null())
        return;

    int no = gtk_combo_box_get_active(GTK_COMBO_BOX(input_style_combo));
    config->write (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_VIEW_TYPE), no);

    no = gtk_combo_box_get_active(GTK_COMBO_BOX(charset_combo));
    config->write (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_CHARHSET), no);

    gboolean stat;
    stat = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(minus_pageup_button));
    config->write (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_PAGE_MINUS), (bool)stat);

    stat = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(bracket_pageup_button));
    config->write (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_PAGE_BRACKET), (bool)stat);

    stat = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(comma_pageup_button));
    config->write (String (SCIM_CONFIG_IMENGINE_SUNPINYIN_USER_PAGE_COMMA), (bool)stat);

    __have_changed = false;
}

bool
query_changed ()
{
    return __have_changed;
}
