/*
 * Copyright (c) 2007 Kov Chai <tchaikov@gmail.com>
 */

#include "sunpinyin_utils.h"
#include "sunpinyin_keycode.h"

using namespace scim;

KeyEvent SunKeyEvent::m_prev_key;

/**
 * translate from SCIM's KeyEvent to SunKeyEvent
 */
SunKeyEvent::SunKeyEvent(const KeyEvent& key)
    :  m_release(false), code(0), value(0), modifier(0)
{
    code = key.code;
    // refer to definition of struct ImeKeyRec in sunpinyin/ime/iiim/cle/ime.h
    value = key.get_ascii_code();
    if (key.is_shift_down())     modifier |= IM_SHIFT_MASK;
    if (key.is_control_down())   modifier |= IM_CTRL_MASK;
    if (key.is_alt_down())       modifier |= IM_ALT_MASK;
    
    if (is_shift(key)) {
        // XXX
        code = IM_VK_SHIFT;
        modifier = IM_ALT_MASK;
    } else if (key.is_key_release())
        m_release = true;
    m_prev_key = key;
}

bool
SunKeyEvent::is_key_release() const
{
    return m_release;
}

// XXX, a dirty hack
// SCIM will not deliver ALT_MASK+VK_SHIFT to process_key_event()
// only a VK_SHIFT+Release is post when the combination key is released.
// But CIMIView::onKeyEvent() switches CN/EN status on Alt+Shift
// 
// If we only switch EN/CN when Alt+Shift is pressed
// following condition statement should be used instead:
//   key.code == SCIM_KEY_Shift_L &&
//   key.is_key_release() &&
//   !key.is_shift_down()
//
//  if (key.is_shift_down())
//  Shift pressed event will also trigger CN/EN switch
bool
SunKeyEvent::is_alt_shift(const KeyEvent& key) const
{
    return ( (key.code == SCIM_KEY_Alt_L &&
              key.is_key_release() &&
              key.is_alt_down()) &&
             (m_prev_key.code == SCIM_KEY_Shift_L &&
              m_prev_key.is_key_release() &&
              m_prev_key.is_alt_down()) );
}

bool
SunKeyEvent::is_shift(const KeyEvent& key) const
{
    return ( (key.code == SCIM_KEY_Shift_L &&
              key.is_shift_down() &&
              key.is_key_release()) &&
             (m_prev_key.code == SCIM_KEY_Shift_L &&
              m_prev_key.mask == SCIM_KEY_NullMask) );
}

bool
SunKeyEvent::is_shift() const
{
    return (code == IM_VK_SHIFT &&
            modifier == IM_ALT_MASK);
}

WideString
wstr_to_widestr(const TWCHAR* wstr)
{
    WideString wide_str;
    copy(wstr, wstr+WCSLEN(wstr)+1, back_inserter(wide_str));
    return wide_str;
}

WideString
wstr_to_widestr(const TWCHAR* wstr, size_t len)
{
    WideString wide_str;
    copy(wstr, wstr+len+1, back_inserter(wide_str));
    return wide_str;
}
